import type { Adapter, AdapterPayload } from 'oidc-provider';
import redis from '../redis';

export class RedisAdapter implements Adapter {
    name: string;
    constructor(name: string) {
        this.name = name;
    }

    key(id: string) {
        return `${this.name}:${id}`;
    }

    async upsert(id: string, payload: AdapterPayload, expiresIn: number) {
        const key = this.key(id);
        await redis.set(key, JSON.stringify(payload), 'EX', expiresIn);
        if (payload && payload.grantId) {
            // 记录grantId到id的映射，便于revokeByGrantId
            await redis.sadd(this.grantKey(payload.grantId), id);
            await redis.expire(this.grantKey(payload.grantId), expiresIn);
        }
        if (payload && payload.userCode) {
            await redis.set(this.userCodeKey(payload.userCode), id, 'EX', expiresIn);
        }
        if (payload && payload.uid) {
            await redis.set(this.uidKey(payload.uid), id, 'EX', expiresIn);
        }
    }

    async find(id: string) {
        const data = await redis.get(this.key(id));
        return data ? JSON.parse(data) : undefined;
    }

    async findByUserCode(userCode: string) {
        const id = await redis.get(this.userCodeKey(userCode));
        return id ? this.find(id) : undefined;
    }

    async findByUid(uid: string) {
        const id = await redis.get(this.uidKey(uid));
        return id ? this.find(id) : undefined;
    }

    async destroy(id: string) {
        const data = await this.find(id);
        await redis.del(this.key(id));
        if (data && data.grantId) {
            await redis.srem(this.grantKey(data.grantId), id);
        }
        if (data && data.userCode) {
            await redis.del(this.userCodeKey(data.userCode));
        }
        if (data && data.uid) {
            await redis.del(this.uidKey(data.uid));
        }
    }

    async revokeByGrantId(grantId: string) {
        const key = this.grantKey(grantId);
        const ids = await redis.smembers(key);
        if (ids && ids.length) {
            await Promise.all(ids.map((id) => this.destroy(id)));
        }
        await redis.del(key);
    }

    async consume(id: string) {
        const key = this.key(id);
        const data = await this.find(id);
        if (data) {
            data.consumed = Math.floor(Date.now() / 1000);
            await redis.set(key, JSON.stringify(data));
        }
    }

    grantKey(grantId: string) {
        return `${this.name}:grant:${grantId}`;
    }
    userCodeKey(userCode: string) {
        return `${this.name}:userCode:${userCode}`;
    }
    uidKey(uid: string) {
        return `${this.name}:uid:${uid}`;
    }
}
